package org.hardknockjava.finale.driver;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.InputStream;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import javax.imageio.ImageIO;

import org.hardknockjava.finale.grid.Clue;
import org.hardknockjava.finale.grid.Grid;

public class Crossword {
    protected Grid grid;
    protected List<Clue> clueList;
    protected BufferedImage image;
   
    public Crossword() throws Exception {
        File inputFile = new File("data", "crossword.zip");
        ZipFile zipFile = new ZipFile(inputFile);
        
        grid = new Grid();
        ZipEntry gridEntry = zipFile.getEntry("data/grid.txt");
        InputStream gridIs = zipFile.getInputStream(gridEntry);
        grid.populate(gridIs);
        
        ZipEntry clueEntry = zipFile.getEntry("data/clues.txt");
        InputStream clueIs = zipFile.getInputStream(clueEntry);
        clueList = Clue.populate(clueIs);
        
        ZipEntry spiritEntry = zipFile.getEntry("data/spirit.jpg");
        InputStream spiritIs = zipFile.getInputStream(spiritEntry);
        image = ImageIO.read(spiritIs);
        
        zipFile.close();
    }
    
    public void writeToPrinter() throws Exception {
        CrosswordPrinter printer = new CrosswordPrinter(this);
        printer.printPuzzle();
    }
    
    public void writeToPdf() throws Exception {
        CrosswordPdfWriter writer = new CrosswordPdfWriter(this);
        writer.printPuzzle();
    }

    public Grid getGrid() {
        return grid;
    }

    public List<Clue> getClueList() {
        return clueList;
    }

    public BufferedImage getImage() {
        return image;
    }

}
