package org.hardknockjava.finale.grid;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/********************************************************************
 * 
 * A Clue has a direction (ACROSS or DOWN), a number, and a text.
 *
 */
public class Clue implements Comparable<Clue> {
    private Integer number;
    private Direction direction;
    private String text;
    private static Pattern cluePattern = Pattern.compile("(\\d*)([AD])\\t(.*)");

    public Clue(Integer number, Direction direction, String text) {
        super();
        this.number = number;
        this.direction = direction;
        this.text = text;
    }

    public Integer getNumber() {
        return number;
    }

    public Direction getDirection() {
        return direction;
    }

    public String getText() {
        return text;
    }

    @Override
    // Clues are sorted in direction, then by number.
    public int compareTo(Clue other) {
        int result = 0;
        result = this.direction.ordinal() - other.direction.ordinal();
        if (result != 0) {
            return result;
        }
        return this.getNumber().intValue() - other.getNumber().intValue();
    }

    /*
     * This method reads the clue file and returns a sorted list of clues. Each line
     * of the clue file is the number, the direction ("A" or "D"), a tab, and the
     * clue text.
     */

    public static List<Clue> populate(File clueFile) throws IOException {
        InputStream stream = new FileInputStream(clueFile);
        return populate(stream);
    }
        
    public static List<Clue> populate(InputStream clueStream) throws IOException {
        List<Clue> clueList = new ArrayList<Clue>();

        BufferedReader rdr = new BufferedReader(new InputStreamReader(clueStream));
        String line = null;
        try {
            while ((line = rdr.readLine()) != null) {
                Matcher m = cluePattern.matcher(line);
                if (m.matches()) {
                    String numberString = m.group(1);
                    String directionString = m.group(2);
                    String textTemp = m.group(3).trim();
                    Integer numberTemp = Integer.parseInt(numberString);
                    Direction directionTemp = (directionString.equalsIgnoreCase("A")) ? Direction.ACROSS
                            : Direction.DOWN;
                    Clue clue = new Clue(numberTemp, directionTemp, textTemp);
                    clueList.add(clue);
                }
            }
        } finally {
            rdr.close();
        }

        return clueList;
    }

}
