package org.hardknockjava.finale.grid;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

/**************************************************************
 * Grid represents the crossword grid.
 *
 */
public class Grid {
    private Square[][] square;
    private int rowCount = 0;
    private int colCount = 0;

    public Square[][] getSquare() {
        return square;
    }

    public int getRowCount() {
        return rowCount;
    }

    public int getColCount() {
        return colCount;
    }

    public void populate(File gridFile) throws IOException {
        InputStream stream = new FileInputStream(gridFile);
        populate(stream);
    }
        
    public void populate(InputStream gridStream) throws IOException {
        List<String> rowList = new ArrayList<String>();
        BufferedReader rdr = new BufferedReader(new InputStreamReader(gridStream));
        try {
            String line = null;
            while ((line = rdr.readLine()) != null) {
                if (!line.trim().isEmpty()) {
                    rowList.add(line);
                }
            }
        } finally {
            rdr.close();
        }

        // Compute the number of rows and columns.
        rowCount = rowList.size();
        for (String row : rowList) {
            if (row.length() > colCount) {
                colCount = row.length();
            }
        }

        // Allocate the grid.
        square = new Square[rowCount][colCount];

        // Populate the grid.
        for (int row = 0; row < rowCount; row++) {
            String line = rowList.get(row);
            for (int column = 0; column < colCount; column++) {
                Character value = null;

                // If the column number is within the line read,
                // and the corresponding character is not a space,
                // assign the character to value. Otherwise value
                // remains null.
                if (column < line.length()) {
                    if (line.charAt(column) != ' ') {
                        value = line.charAt(column);
                    }
                }
                Square tempSquare = new Square(value, row, column);
                square[row][column] = tempSquare;
            }
        }

        // Assign numbers to the squares in the grid. A square
        // gets the next number if:
        // it is not blank;
        // it is either in the top row, or directly below
        // or to the right of a blank.
        int nextNumber = 0;
        for (int row = 0; row < rowCount; row++) {
            for (int column = 0; column < colCount; column++) {
                Square current = square[row][column];

                // If the square is blank, it doesn't get a number.
                if (current.getValue() == null) {
                    continue;
                }

                boolean assign = false;

                // If the row number is 0 or the square above
                // is blank, assign a number.
                if (row == 0) {
                    assign = true;
                } else {
                    Square above = square[row - 1][column];
                    if (above.getValue() == null) {
                        assign = true;
                    }
                }

                // If the column is zero or the square to the left
                // is blank, assign a number.
                if (column == 0) {
                    assign = true;
                } else {
                    Square left = square[row][column - 1];
                    if (left.getValue() == null) {
                        assign = true;
                    }
                }

                // If assign == true, assign the next number
                // to the square.
                if (assign) {
                    current.setClueNumber(++nextNumber);
                }
            }
        }
    }
}
